/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { FloorScene } from "./FloorScene";
import { ZoneItem } from "./items/ZoneItem";
import { Zone } from "./types/Zone";

/**
 * 业务空间
 *
 * @author 郝建龙
 */
export class ZoneScene extends FloorScene {
    /** 业务空间list   */
    zoneList: ZoneItem[] = [];
    /** 业务空间是否可选    */
    _isZoneSelectable: boolean = true;
    get isZoneSelectable(): boolean {
        return this._isZoneSelectable;
    } // Get isZoneSelectable
    set isZoneSelectable(v: boolean) {
        if (this._isZoneSelectable === v) {
            return;
        }
        this._isZoneSelectable = v;
        this.zoneList.map(
            (t: ZoneItem): ZoneItem => {
                t.selectable = this._isZoneSelectable;
                if (!t.selectable) {
                    t.selected = false;
                }
                return t;
            }
        );
    } // Set isZoneSelectable

    /**
     *  添加业务空间到scene 中
     *
     *  @param  zone   业务空间list
     */
    addZoneList(zone: Zone[]): void {
        zone.forEach((t): void => {
            this.addZone(t);
        });
    } // Function addZoneList()

    /**
     *  添加业务空间到scene 中
     *
     *  @param  zone   业务空间
     */
    addZone(zone: Zone): void {
        let item = new ZoneItem(null, zone);
        item.selectable = this.isZoneSelectable;
        this.zoneList.push(item);
        this.addItem(item);
    } // Function addZone()

    /**
     *  清空选中的业务空间
     *
     */
    clearZoneSelection(): void {
        this.zoneList.map(
            (t): ZoneItem => {
                t.selected = false;
                return t;
            }
        );
    } // Function clearZoneSelection()

    /**
     * 删除所有业务空间
     */
    removeAllZone(): void {
        if (this.zoneList.length) {
            this.zoneList.forEach((t): void => {
                this.removeItem(t);
            });
            this.zoneList = [];
        }
    } // Function removeAllZone()

    /**
     * 获取选中的业务空间
     */
    getSelectedZone(): ZoneItem[] {
        let arr: ZoneItem[] = [];
        this.zoneList.forEach((t): void => {
            if (t.selected) {
                arr.push(t);
            }
        });
        return arr;
    } // Function getSelectedZone()
} // Class ZoneScene
